#ifndef PYTHONIC_INCLUDE_NUMPY_FMAX_REDUCE_HPP
#define PYTHONIC_INCLUDE_NUMPY_FMAX_REDUCE_HPP

#define UFUNC_NAME fmax
#define UFUNC_INAME imax
#include "pythonic/include/numpy/ufunc_reduce.hpp"
#undef UFUNC_NAME
#undef UFUNC_INAME

#endif
