#ifndef PYTHONIC_INCLUDE_NUMPY_AMIN_HPP
#define PYTHONIC_INCLUDE_NUMPY_AMIN_HPP

#include "pythonic/include/numpy/min.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{
  USING_FUNCTOR(amin, min);
}
PYTHONIC_NS_END

#endif
