#ifndef PYTHONIC_INCLUDE_OPERATOR_IDIV_HPP
#define PYTHONIC_INCLUDE_OPERATOR_IDIV_HPP

#define OPERATOR_NAME idiv
#define OPERATOR_SYMBOL /
#define OPERATOR_ISYMBOL /=
#include "pythonic/include/operator_/icommon.hpp"

#endif
