#ifndef PYTHONIC_INCLUDE_NUMPY_NEWAXIS_HPP
#define PYTHONIC_INCLUDE_NUMPY_NEWAXIS_HPP

#include "pythonic/include/types/NoneType.hpp"

PYTHONIC_NS_BEGIN

namespace numpy
{

  auto const &newaxis = builtins::None;
}
PYTHONIC_NS_END

#endif
