#ifndef PYTHONIC_OPERATOR_ISUB_HPP
#define PYTHONIC_OPERATOR_ISUB_HPP

#include "pythonic/include/operator_/isub.hpp"

#define OPERATOR_NAME isub
#define OPERATOR_SYMBOL -
#define OPERATOR_ISYMBOL -=

#include "pythonic/operator_/icommon.hpp"

#endif
