#ifndef PYTHONIC_INCLUDE_BUILTIN_BASEEXCEPTION_HPP
#define PYTHONIC_INCLUDE_BUILTIN_BASEEXCEPTION_HPP

#include "pythonic/include/types/exceptions.hpp"

PYTHONIC_NS_BEGIN

namespace builtins
{

  PYTHONIC_EXCEPTION_DECL(BaseException)
}
PYTHONIC_NS_END

#endif
