#ifndef PYTHONIC_BUILTIN_DEPRECATIONWARNING_HPP
#define PYTHONIC_BUILTIN_DEPRECATIONWARNING_HPP

#include "pythonic/include/builtins/DeprecationWarning.hpp"

#include "pythonic/types/exceptions.hpp"

PYTHONIC_NS_BEGIN

namespace builtins
{

  PYTHONIC_EXCEPTION_IMPL(DeprecationWarning)
}
PYTHONIC_NS_END

#endif
